"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../util");
var di_1 = require("../../di");
var UniversalComponent = /** @class */ (function (_super) {
    __extends(UniversalComponent, _super);
    /**
     * @stable [02.12.2019]
     * @param {TProps} props
     */
    function UniversalComponent(props) {
        var _this = _super.call(this, props) || this;
        _this.plugins = [];
        _this.selfRef = React.createRef();
        _this.defaultUiFactory = { makeIcon: function () { return null; } };
        _this.initPlugins();
        util_1.patchRenderMethod(_this);
        return _this;
    }
    /**
     * @stable [22.09.2019]
     */
    UniversalComponent.prototype.componentDidMount = function () {
        this.plugins.forEach(function (plugin) { return util_1.isFn(plugin.componentDidMount) && plugin.componentDidMount(); });
    };
    /**
     * @stable [22.09.2019]
     */
    UniversalComponent.prototype.componentWillUnmount = function () {
        this.plugins.forEach(function (plugin) { return util_1.isFn(plugin.componentWillUnmount) && plugin.componentWillUnmount(); });
    };
    /**
     * @stable [23.10.2019]
     * @param {Readonly<TProps extends IUniversalComponentProps>} prevProps
     * @param {Readonly<TState>} prevState
     */
    UniversalComponent.prototype.getSnapshotBeforeUpdate = function (prevProps, prevState) {
        this.plugins.forEach(function (plugin) {
            return util_1.isFn(plugin.getSnapshotBeforeUpdate) && plugin.getSnapshotBeforeUpdate(prevProps, prevState);
        });
        return null;
    };
    /**
     * @stable [23.04.2018]
     * @param {Readonly<TProps>} prevProps
     * @param {Readonly<TState>} prevState
     * @param {never} prevContext
     */
    UniversalComponent.prototype.componentDidUpdate = function (prevProps, prevState, prevContext) {
        this.plugins.forEach(function (plugin) { return util_1.isFn(plugin.componentDidUpdate) && plugin.componentDidUpdate(prevProps, prevState, prevContext); });
    };
    /**
     * @stable [01.12.2018]
     * @returns {Element}
     */
    UniversalComponent.prototype.getSelf = function () {
        return this.selfRef.current || this.refs.self; // TODO any
    };
    Object.defineProperty(UniversalComponent.prototype, "environment", {
        /**
         * @react-native-compatible
         * @stable [08.10.2019]
         * @returns {IEnvironment}
         */
        get: function () {
            return di_1.getEnvironment();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "eventManager", {
        /**
         * @react-native-compatible
         * @stable [08.10.2019]
         * @returns {IEventManager}
         */
        get: function () {
            return di_1.getEventManager();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "fieldConverter", {
        /**
         * @react-native-compatible
         * @stable [09.01.2020]
         * @returns {IFieldConverter}
         */
        get: function () {
            return di_1.getFieldConverter();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "placeApi", {
        /**
         * @react-native-compatible
         * @stable [09.01.2020]
         * @returns {IPlaceApi}
         */
        get: function () {
            return di_1.getPlaceApi();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "asyncLibManager", {
        /**
         * @react-native-compatible
         * @stable [09.01.2020]
         * @returns {IAsyncLibManager}
         */
        get: function () {
            return di_1.getAsyncLibManager();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "t", {
        /**
         * @reactNativeCompatible
         * @stable [22.09.2018]
         * @returns {TranslatorT}
         */
        get: function () {
            return di_1.getTranslator();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "settings", {
        /**
         * @reactNativeCompatible
         * @stable [29.07.2019]
         * @returns {ISettingsEntity}
         */
        get: function () {
            return di_1.getSettings();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "databaseStorage", {
        /**
         * @reactNativeCompatible
         * @stable [29.07.2019]
         * @returns {IStorage}
         */
        get: function () {
            return di_1.getDatabaseStorage();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "dc", {
        /**
         * @stable [13.05.2018]
         * @returns {IDateConverter}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.DateConverter);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "nc", {
        /**
         * @reactNativeCompatible
         * @stable [22.09.2019]
         * @returns {INumberConverter}
         */
        get: function () {
            return di_1.getNumberConverter();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "uiFactory", {
        /**
         * @stable [18.05.2018]
         * @returns {IUIFactory}
         */
        get: function () {
            return util_1.nvl(di_1.getUiFactory(), this.defaultUiFactory);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "domAccessor", {
        /**
         * @stable [01.12.2018]
         * @returns {IDomAccessor}
         */
        get: function () {
            return di_1.staticInjector(di_1.DI_TYPES.DomAccessor);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalComponent.prototype, "transport", {
        /**
         * @reactNativeCompatible
         * @stable [15.09.2019]
         * @returns {ITransport}
         */
        get: function () {
            return di_1.getTransport();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [18.06.2019]
     * @param {IUniversalPluginCtor | IUniversalPlugin} pluginObject
     */
    UniversalComponent.prototype.registerPlugin = function (pluginObject) {
        if (R.isNil(pluginObject)) {
            return;
        }
        this.plugins.push(util_1.isFn(pluginObject)
            ? Reflect.construct(pluginObject, [this])
            : pluginObject);
    };
    /**
     * @stable [18.06.2019]
     */
    UniversalComponent.prototype.initPlugins = function () {
        var _this = this;
        var plugins = this.uiPlugins;
        if (R.isNil(plugins)) {
            return;
        }
        var dynamicPluginFactory = plugins.get(this.constructor);
        if (dynamicPluginFactory) {
            this.registerPlugin(dynamicPluginFactory(this));
        }
        [].concat(this.props.plugins || []).forEach(function (plugin) { return _this.registerPlugin(plugin); });
    };
    Object.defineProperty(UniversalComponent.prototype, "uiPlugins", {
        /**
         * @reactNativeCompatible
         * @stable [21.08.2019]
         * @returns {Map<IUniversalComponentCtor, UniversalPluginFactoryT>}
         */
        get: function () {
            return di_1.getUiPlugins();
        },
        enumerable: true,
        configurable: true
    });
    return UniversalComponent;
}(React.PureComponent));
exports.UniversalComponent = UniversalComponent;
//# sourceMappingURL=universal.component.js.map