"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var base_component_1 = require("../base/base.component");
var definition_1 = require("../../definition");
var grid_1 = require("../grid");
var util_1 = require("../../util");
var Calendar = /** @class */ (function (_super) {
    __extends(Calendar, _super);
    /**
     * @stable [04.01.2020]
     * @param {ICalendarProps} props
     */
    function Calendar(props) {
        var _this = _super.call(this, props) || this;
        _this.onClick = _this.onClick.bind(_this);
        _this.getCellElement = _this.getCellElement.bind(_this);
        _this.getColumnClassName = _this.getColumnClassName.bind(_this);
        return _this;
    }
    /**
     * @stable [04.01.2020]
     * @returns {JSX.Element}
     */
    Calendar.prototype.render = function () {
        var _this = this;
        var props = this.props;
        var calendar = this.calendarEntity;
        var listEntity = __assign({}, definition_1.INITIAL_LIST_ENTITY, { data: calendar.days });
        var columns = calendar.daysLabels.map(function (day, index) { return ({
            title: day,
            index: index,
            align: 'center',
            columnClassName: _this.getColumnClassName,
            renderer: _this.getCellElement,
            onColumnContentClick: _this.onClick,
        }); });
        return (React.createElement(grid_1.Grid, __assign({}, definition_1.NOT_SELECTABLE_GRID_ENTITY, { headerRendered: false, highlightOdd: false }, props.gridConfiguration, { columnsConfiguration: columns, className: util_1.joinClassName('rac-calendar', props.className) }, listEntity)));
    };
    /**
     * @stable [04.01.2020]
     * @param {IGridColumnProps<ICalendarDayEntity>} payload
     * @returns {string}
     */
    Calendar.prototype.getColumnClassName = function (payload) {
        var entity = this.asCalendarDayEntity(payload);
        var isDaySelected = this.isDaySelected(entity);
        return util_1.joinClassName(entity.current && 'rac-calendar-current-day', entity.today && 'rac-calendar-today', isDaySelected && 'rac-calendar-selected-day');
    };
    /**
     * @stable [07.01.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    Calendar.prototype.isDaySelected = function (entity) {
        var _a = this.props, isSelected = _a.isSelected, _b = _a.selectedDays, selectedDays = _b === void 0 ? [] : _b;
        return util_1.isFn(isSelected)
            ? isSelected(entity)
            : selectedDays.includes(entity.day) && entity.current;
    };
    /**
     * @stable [04.01.2020]
     * @param {ICalendarWeekEntity} item
     * @param {IGridColumnProps<ICalendarDayEntity>} column
     * @returns {JSX.Element}
     */
    Calendar.prototype.getCellElement = function (item, column) {
        var props = this.props;
        var weekDayEntity = item[column.index];
        if (props.showOnlyCurrentDays && !weekDayEntity.current) {
            return null;
        }
        if (util_1.isFn(props.renderer)) {
            return props.renderer(weekDayEntity);
        }
        return React.createElement(React.Fragment, null, weekDayEntity.date.getDate());
    };
    /**
     * @stable [04.01.2020]
     * @param {IGridColumnProps} payload
     */
    Calendar.prototype.onClick = function (payload) {
        var props = this.props;
        if (util_1.isFn(props.onSelect)) {
            props.onSelect(this.asCalendarDayEntity(payload));
        }
    };
    /**
     * @stable [04.01.2020]
     * @param {IGridColumnProps} payload
     * @returns {ICalendarDayEntity}
     */
    Calendar.prototype.asCalendarDayEntity = function (payload) {
        return payload.entity[payload.index];
    };
    Object.defineProperty(Calendar.prototype, "calendarEntity", {
        /**
         * @stable [04.01.2020]
         * @returns {ICalendarEntity}
         */
        get: function () {
            var props = this.props;
            return props.calendarEntity || this.dc.asCalendar({ useSyntheticCalendar: true });
        },
        enumerable: true,
        configurable: true
    });
    Calendar.defaultProps = {
        showOnlyCurrentDays: false,
    };
    return Calendar;
}(base_component_1.BaseComponent));
exports.Calendar = Calendar;
//# sourceMappingURL=calendar.component.js.map