"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOM = require("react-dom");
var button_1 = require("../button");
var util_1 = require("../../util");
var plugin_1 = require("../plugin");
var basic_component_1 = require("../base/basic.component");
var base_component_1 = require("../base/base.component");
var Dialog = /** @class */ (function (_super) {
    __extends(Dialog, _super);
    /**
     * @stable [06.01.2020]
     * @param {TProps} props
     */
    function Dialog(props) {
        var _this = _super.call(this, props) || this;
        _this.onAcceptClick = _this.onAcceptClick.bind(_this);
        _this.onCloseClick = _this.onCloseClick.bind(_this);
        _this.onDialogScrimClick = _this.onDialogScrimClick.bind(_this);
        _this.state = { opened: false };
        return _this;
    }
    /**
     * @stable [05.01.2020]
     * @returns {React.ReactNode}
     */
    Dialog.prototype.render = function () {
        var _this = this;
        return util_1.orNull(this.state.opened, function () { return (ReactDOM.createPortal(React.createElement("div", { ref: _this.selfRef, className: _this.getDialogClassName() },
            _this.dialogScrimElement,
            React.createElement("div", { className: 'rac-dialog__body-wrapper rac-absolute rac-full-size' }, _this.dialogBodyElement)), _this.domAccessor.getDocumentBodyElement())); });
    };
    /**
     * @stable [06.01.2020]
     * @param {IActivateDialogConfigEntity} payload
     */
    Dialog.prototype.activate = function (payload) {
        var _this = this;
        this.setState({ opened: true }, function () {
            var _a = payload || {}, onActivate = _a.onActivate, onDeactivate = _a.onDeactivate;
            if (util_1.isFn(onDeactivate)) {
                _this.onDeactivateCallback = onDeactivate;
            }
            if (util_1.isFn(onActivate)) {
                onActivate.call(_this);
            }
            var props = _this.props;
            if (util_1.isFn(props.onActivate)) {
                props.onActivate();
            }
        });
    };
    /**
     * @stable [06.01.2020]
     */
    Dialog.prototype.accept = function () {
        this.onAcceptClick();
    };
    /**
     * @stable [05.01.2020]
     * @returns {string}
     */
    Dialog.prototype.getDialogClassName = function () {
        return util_1.joinClassName('rac-dialog', this.props.className);
    };
    /**
     * @stable [06.01.2020]
     */
    Dialog.prototype.onAcceptClick = function () {
        var props = this.props;
        this.doClose(function () {
            if (util_1.isFn(props.onAccept)) {
                props.onAccept();
            }
        });
    };
    /**
     * @stable [05.01.2020]
     */
    Dialog.prototype.onCloseClick = function () {
        var props = this.props;
        this.doClose(function () {
            if (util_1.isFn(props.onClose)) {
                props.onClose();
            }
        });
    };
    /**
     * @stable [05.01.2020]
     */
    Dialog.prototype.onDialogScrimClick = function () {
        this.doClose();
    };
    /**
     * @stable [05.01.2020]
     * @param {() => void} callback
     */
    Dialog.prototype.doClose = function (callback) {
        var _this = this;
        var props = this.props;
        this.setState({ opened: false }, function () {
            if (util_1.isFn(_this.onDeactivateCallback)) {
                _this.onDeactivateCallback.call(_this);
                _this.onDeactivateCallback = null;
            }
            if (util_1.isFn(props.onDeactivate)) {
                props.onDeactivate();
            }
            if (util_1.isFn(callback)) {
                callback.call(_this);
            }
        });
    };
    Object.defineProperty(Dialog.prototype, "footerElement", {
        /**
         * @stable [24.03.2019]
         * @returns {JSX.Element}
         */
        get: function () {
            var _this = this;
            var props = this.props;
            return (util_1.orNull(this.closable || this.acceptable, function () { return (React.createElement("div", { className: 'rac-dialog__actions' },
                util_1.orNull(_this.closable, function () { return (React.createElement(button_1.Button, { icon: 'close', full: true, disabled: _this.isCloseButtonDisabled, onClick: _this.onCloseClick }, _this.t(props.closeText || _this.settings.messages.DIALOG_CANCEL))); }),
                util_1.orNull(_this.acceptable, function () { return (React.createElement(button_1.Button, { icon: 'ok-filled', full: true, raised: true, disabled: _this.isAcceptButtonDisabled, onClick: _this.onAcceptClick }, _this.t(props.acceptText || _this.settings.messages.DIALOG_ACCEPT))); }))); }));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "dialogScrimElement", {
        /**
         * @stable [05.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement("div", __assign({ className: 'rac-dialog__scrim rac-absolute rac-full-size' }, util_1.handlerPropsFactory(this.onDialogScrimClick))));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "dialogBodyElement", {
        /**
         * @stable [05.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            return (React.createElement("div", { className: 'rac-dialog__body' },
                this.bodyElement,
                this.footerElement));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "bodyElement", {
        /**
         * @stable [06.01.2020]
         * @returns {JSX.Element}
         */
        get: function () {
            var props = this.props;
            return (React.createElement(React.Fragment, null,
                props.title && (React.createElement("div", { className: 'rac-dialog__body-title' }, this.t(props.title))),
                props.children && (React.createElement(basic_component_1.BasicComponent, { className: 'rac-dialog__body-content', plugins: plugin_1.PerfectScrollPlugin }, props.children))));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "acceptable", {
        /**
         * @stable [06.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return this.props.acceptable !== false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "closable", {
        /**
         * @stable [06.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return this.props.closable !== false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "isCloseButtonDisabled", {
        /**
         * @stable [06.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return this.props.closeDisabled === true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Dialog.prototype, "isAcceptButtonDisabled", {
        /**
         * @stable [06.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return this.props.acceptDisabled === true;
        },
        enumerable: true,
        configurable: true
    });
    return Dialog;
}(base_component_1.BaseComponent));
exports.Dialog = Dialog;
//# sourceMappingURL=dialog.component.js.map