"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var definition_1 = require("../../../definition");
var textfield_1 = require("../textfield");
var button_1 = require("../../button");
var calendar_1 = require("../../calendar");
var dialog_1 = require("../../dialog");
var numberfield_1 = require("../numberfield");
var DateField = /** @class */ (function (_super) {
    __extends(DateField, _super);
    /**
     * @stable [07.01.2020]
     * @param {TProps} props
     */
    function DateField(props) {
        var _this = _super.call(this, props) || this;
        _this.dialogRef = React.createRef();
        _this.isDaySelected = _this.isDaySelected.bind(_this);
        _this.onAccept = _this.onAccept.bind(_this);
        _this.onOk = _this.onOk.bind(_this);
        _this.onChangeYear = _this.onChangeYear.bind(_this);
        _this.onDialogDeactivate = _this.onDialogDeactivate.bind(_this);
        _this.openDialog = _this.openDialog.bind(_this);
        _this.setNextMonth = _this.setNextMonth.bind(_this);
        _this.setPreviousMonth = _this.setPreviousMonth.bind(_this);
        _this.defaultActions = [
            { type: definition_1.FieldActionTypesEnum.CALENDAR, onClick: _this.openDialog }
        ].concat(_this.defaultActions);
        return _this;
    }
    /**
     * @stable [07.01.2020]
     * @param {DateTimeLikeTypeT} currentRawValue
     */
    DateField.prototype.onChangeManually = function (currentRawValue) {
        _super.prototype.onChangeManually.call(this, this.serializeValue(currentRawValue));
    };
    /**
     * @stable [09.01.2020]
     * @returns {JSX.Element}
     */
    DateField.prototype.getInputAttachmentElement = function () {
        var _this = this;
        var _a = this.state, year = _a.year, dialogOpened = _a.dialogOpened;
        var props = this.props;
        return util_1.orNull(dialogOpened, // To improve a performance
        function () {
            return (React.createElement(dialog_1.Dialog, { ref: _this.dialogRef, acceptable: false, closable: false, className: util_1.joinClassName(props.dialogClassName, 'rac-calendar-dialog', 'rac-date-field__calendar-dialog'), onDeactivate: _this.onDialogDeactivate },
                React.createElement("div", { className: 'rac-calendar-dialog__range-explorer' },
                    React.createElement(button_1.Button, { icon: 'back', mini: true, onClick: _this.setPreviousMonth }),
                    React.createElement("span", { className: 'rac-calendar-dialog__range-explorer-date' }, _this.dateToDisplay()),
                    React.createElement(button_1.Button, { icon: 'forward', mini: true, onClick: _this.setNextMonth })),
                React.createElement(calendar_1.Calendar, __assign({ showOnlyCurrentDays: true, gridConfiguration: { headerRendered: true } }, props.calendarConfiguration, { calendarEntity: _this.calendarEntity, isSelected: _this.isDaySelected, onSelect: _this.onAccept })),
                React.createElement("div", { className: 'rac-calendar-dialog__footer' },
                    React.createElement(numberfield_1.NumberField, { value: year, full: false, autoFocus: true, keepChanges: true, errorMessageRendered: false, pattern: _this.dateTimeSettings.uiYearPattern, mask: _this.dateTimeSettings.uiYearMask, placeholder: _this.selectedYearPlaceholder, onChange: _this.onChangeYear }),
                    React.createElement(button_1.Button, { text: _this.settings.messages.OK, onClick: _this.onOk }))));
        });
    };
    /**
     * @stable [09.01.2020]
     * @param {IBaseEvent} event
     */
    DateField.prototype.onClick = function (event) {
        _super.prototype.onClick.call(this, event);
        if (this.isFocusPrevented) {
            this.openDialog();
        }
    };
    /**
     * @stable [07.01.2019]
     * @param {AnyT} value
     * @returns {string}
     */
    DateField.prototype.decorateValueBeforeDisplaying = function (value) {
        return this.serializeValue(value);
    };
    /**
     * @stable [02.05.2019]
     * @returns {string}
     */
    DateField.prototype.getFieldClassName = function () {
        return util_1.joinClassName(_super.prototype.getFieldClassName.call(this), 'rac-date-field');
    };
    /**
     * @stable [07.01.2020]
     * @returns {string}
     */
    DateField.prototype.getFieldPattern = function () {
        var _this = this;
        return util_1.orNull(!this.isDisplayValueDefined, function () { return _super.prototype.getFieldPattern.call(_this) || _this.dateTimeSettings.uiDatePattern; });
    };
    /**
     * @stable [07.01.2020]
     * @returns {Array<string | RegExp>}
     */
    DateField.prototype.getFieldMask = function () {
        var _this = this;
        return util_1.orNull(!this.isDisplayValueDefined, function () { return _super.prototype.getFieldMask.call(_this) || _this.dateTimeSettings.uiDateMask; });
    };
    Object.defineProperty(DateField.prototype, "fieldFormat", {
        /**
         * @stable [07.01.2020]
         * @returns {string}
         */
        get: function () {
            return util_1.nvl(this.props.format, this.dateTimeSettings.uiDateFormat);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [08.01.2020]
     */
    DateField.prototype.setPreviousMonth = function () {
        this.changeMonth(-1);
    };
    /**
     * @stable [08.01.2020]
     */
    DateField.prototype.setNextMonth = function () {
        this.changeMonth(1);
    };
    /**
     * @stable [08.01.2020]
     * @param {number} duration
     */
    DateField.prototype.changeMonth = function (duration) {
        this.setState({ date: this.dc.addMonthsAsDate({ date: this.calendarDate, duration: duration }) });
    };
    /**
     * @stable [08.01.2020]
     * @param {number} year
     */
    DateField.prototype.onChangeYear = function (year) {
        var props = this.props;
        var date = this.dc.fromDayOfYearEntityAsDate({ year: year }, { date: this.selectedDate });
        if (this.dc.isDateBelongToDatesRange({ date: date, minDate: props.minDate, maxDate: props.maxDate })) {
            this.setState({ year: year, date: date });
        }
        else {
            this.setState({ year: year });
        }
    };
    /**
     * @stable [14.01.2020]
     */
    DateField.prototype.onOk = function () {
        this.onAccept();
    };
    /**
     * @stable [09.01.2020]
     * @param {ICalendarDayEntity} calendarDayEntity
     */
    DateField.prototype.onAccept = function (calendarDayEntity) {
        var _this = this;
        this.onDialogClose(function () {
            if (!R.isNil(calendarDayEntity)) {
                _this.onChangeManually(calendarDayEntity.date);
            }
            if (!_this.isFocusPrevented) {
                _this.setFocus();
            }
        });
    };
    /**
     * @stable [09.01.2020]
     */
    DateField.prototype.onDialogDeactivate = function () {
        this.onDialogClose();
    };
    /**
     * @stable [09.01.2020]
     * @param {() => void} callback
     */
    DateField.prototype.onDialogClose = function (callback) {
        this.setState({ dialogOpened: false, date: definitions_interface_1.UNDEF, year: definitions_interface_1.UNDEF }, callback);
    };
    /**
     * @stable [24.02.2019]
     */
    DateField.prototype.openDialog = function () {
        var _this = this;
        this.setState({ dialogOpened: true }, function () { return _this.dialog.activate(); });
    };
    /**
     * @stable [08.01.2020]
     * @param {ICalendarDayEntity} entity
     * @returns {boolean}
     */
    DateField.prototype.isDaySelected = function (entity) {
        return this.dc.compare(entity.date, this.selectedDate) === 0;
    };
    Object.defineProperty(DateField.prototype, "selectedDate", {
        /**
         * @stable [07.01.2020]
         * @returns {Date}
         */
        get: function () {
            return this.dc.asDate({ date: this.value, inputFormat: this.fieldFormat });
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [07.01.2020]
     * @param {DateTimeLikeTypeT} value
     * @returns {string}
     */
    DateField.prototype.serializeValue = function (value) {
        return this.dc.dateAsString({
            date: value,
            inputFormat: this.fieldFormat,
            outputFormat: this.fieldFormat,
        });
    };
    /**
     * @stable [08.01.2020]
     * @returns {string}
     */
    DateField.prototype.dateToDisplay = function () {
        return this.dc.dateAsString({ date: this.calendarDate, outputFormat: this.props.headerFormat });
    };
    Object.defineProperty(DateField.prototype, "calendarEntity", {
        /**
         * @stable [07.01.2020]
         * @returns {ICalendarEntity}
         */
        get: function () {
            return this.dc.asCalendar({ date: this.calendarDate });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "calendarDate", {
        /**
         * @stable [08.01.2020]
         * @returns {Date}
         */
        get: function () {
            return this.state.date || this.selectedDate || this.dc.asDayOfYear();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "selectedYearPlaceholder", {
        /**
         * @stable [08.01.2020]
         * @returns {string}
         */
        get: function () {
            var _this = this;
            return util_1.ifNotNilThanValue(this.selectedDate, function (selectedDate) { return String(_this.dc.asDayOfYearEntity({ date: selectedDate }).year); }, definitions_interface_1.UNDEF_SYMBOL);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "dateTimeSettings", {
        /**
         * stable [07.01.2020]
         * @returns {IDateTimeSettingsEntity}
         */
        get: function () {
            return this.settings.dateTime || {};
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DateField.prototype, "dialog", {
        /**
         * @stable [24.02.2019]
         * @returns {Dialog}
         */
        get: function () {
            return this.dialogRef.current;
        },
        enumerable: true,
        configurable: true
    });
    DateField.defaultProps = {
        headerFormat: 'MMMM YYYY',
        preventFocus: true,
    };
    return DateField;
}(textfield_1.BaseTextField));
exports.DateField = DateField;
//# sourceMappingURL=date-field.component.js.map