"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var Printf = require("sprintf-js");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../../util");
var definitions_interface_1 = require("../../../definitions.interface");
var field_interface_1 = require("./field.interface");
var definition_1 = require("../../../definition");
var universal_component_1 = require("../../base/universal.component");
var UniversalField = /** @class */ (function (_super) {
    __extends(UniversalField, _super);
    /**
     * @stable [17.06.2018]
     * @param {TProps} props
     */
    function UniversalField(props) {
        var _this = _super.call(this, props) || this;
        _this.onChange = _this.onChange.bind(_this);
        _this.onChangeManually = _this.onChangeManually.bind(_this);
        _this.onFocus = _this.onFocus.bind(_this);
        _this.onBlur = _this.onBlur.bind(_this);
        _this.onKeyUp = _this.onKeyUp.bind(_this);
        _this.onKeyDown = _this.onKeyDown.bind(_this);
        _this.closeVirtualKeyboard = _this.closeVirtualKeyboard.bind(_this);
        _this.onCloseVirtualKeyboard = _this.onCloseVirtualKeyboard.bind(_this);
        _this.state = {};
        if (_this.isKeyboardUsed && _this.isSyntheticCursorUsed) {
            _this.caretBlinkingTask = new util_1.DelayedTask(_this.setCaretVisibility.bind(_this), props.caretBlinkingFrequencyTimeout || UniversalField.DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT, true);
        }
        return _this;
    }
    /**
     * @stable [20.08.2018]
     */
    UniversalField.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        // Need to invoke a user validator if it exists (After F5, etc...)
        this.validateValueAndSetCustomValidity(this.value);
        if (this.props.autoFocus) {
            this.setFocus();
        }
    };
    /**
     * @stable [04.09.2018]
     * @param {TProps} prevProps
     * @param {TState} prevState
     */
    UniversalField.prototype.componentDidUpdate = function (prevProps, prevState) {
        _super.prototype.componentDidUpdate.call(this, prevProps, prevState);
        var keyboardUsed = this.isKeyboardUsed;
        if (keyboardUsed && this.isKeyboardOpen() && !R.equals(this.value, prevProps.value)) {
            this.refreshCaretPosition();
        }
        if (util_1.isKeyboardUsed(prevProps) && !keyboardUsed) {
            this.closeVirtualKeyboard();
        }
    };
    /**
     * @stable [17.06.2018]
     */
    UniversalField.prototype.resetError = function () {
        this.validateField(definitions_interface_1.CLEAR_DIRTY_CHANGES_VALUE);
    };
    /**
     * @stable [20.08.2018]
     */
    UniversalField.prototype.clearValue = function () {
        this.setFocus(); // UX
        if (this.isValuePresent) {
            this.onChangeManually(this.emptyValue);
        }
        var props = this.props;
        if (util_1.isFn(props.onClear)) {
            props.onClear();
        }
    };
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    UniversalField.prototype.onChange = function (event) {
        this.onChangeValue(this.getRawValueFromEvent(event));
    };
    /**
     * @stable [07.01.2018]
     * @param {AnyT} currentRawValue
     */
    UniversalField.prototype.onChangeManually = function (currentRawValue) {
        if (this.props.preventManualChanges !== true) {
            this.onChangeValue(currentRawValue);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyBackspace = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyBackspace)) {
            props.onKeyBackspace(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyUp = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyUp)) {
            props.onKeyUp(event);
        }
    };
    /**
     * @stable [03.09.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyDown = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyDown)) {
            props.onKeyDown(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyEnter = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyEnter)) {
            props.onKeyEnter(event);
        }
    };
    /**
     * @stable [04.09.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyTab = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyTab)) {
            props.onKeyTab(event);
        }
    };
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    UniversalField.prototype.onKeyEscape = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyEscape)) {
            props.onKeyEscape(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyArrowDown = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyArrowDown)) {
            props.onKeyArrowDown(event);
        }
    };
    /**
     * @stable [18.06.2018]
     * @param {IKeyboardEvent} event
     */
    UniversalField.prototype.onKeyArrowUp = function (event) {
        var props = this.props;
        if (util_1.isFn(props.onKeyArrowUp)) {
            props.onKeyArrowUp(event);
        }
    };
    Object.defineProperty(UniversalField.prototype, "value", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            var value = this.props.value;
            return this.isValueDefined(value) ? value : this.defaultValue;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [29.10.2019]
     * @param {boolean} usePrintf
     * @param {AnyT} args
     * @returns {string}
     */
    UniversalField.prototype.buildDisplayMessage = function (usePrintf) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return usePrintf
            ? Printf.sprintf.apply(Printf, [this.t(this.props.displayMessage)].concat(args)) : definition_1.FIELD_DISPLAY_EMPTY_VALUE;
    };
    Object.defineProperty(UniversalField.prototype, "isValueNotPresent", {
        /**
         * @stable [30.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return !this.isValuePresent;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isRequired", {
        /**
         * @stable [29.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isRequired(this.props);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [05.10.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldInvalid = function () {
        return !R.isNil(this.error);
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    UniversalField.prototype.validateValueAndSetCustomValidity = function (value) {
        var props = this.props;
        var error = field_interface_1.FIELD_EMPTY_ERROR_VALUE;
        this.setNativeInputValidity('');
        if (this.isInputValid()) {
            error = util_1.isFn(props.validate) ? props.validate(value) : error;
            if (R.isNil(error)) {
                error = this.validateValue(value); // The custom internal validator
            }
            if (!R.isNil(error)) {
                this.setNativeInputValidity(error);
            }
        }
        else {
            error = this.getNativeInputValidationMessage();
        }
        return error;
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} value
     * @returns {string}
     */
    UniversalField.prototype.validateValue = function (value) {
        return field_interface_1.FIELD_EMPTY_ERROR_VALUE;
    };
    /**
     * @stable [03.09.2018]
     * @returns {boolean}
     */
    UniversalField.prototype.isInputValid = function () {
        return true;
    };
    /**
     * @stable [31.07.2018]
     * @returns {string}
     */
    UniversalField.prototype.getNativeInputValidationMessage = function () {
        return field_interface_1.FIELD_EMPTY_ERROR_VALUE;
    };
    /**
     * @stable [31.07.2018]
     * @param {string} error
     */
    UniversalField.prototype.setNativeInputValidity = function (error) {
        // Do nothing
    };
    /**
     * @stable [17.06.2018]
     * @returns {Array<string | RegExp>}
     */
    UniversalField.prototype.getFieldMask = function () {
        return this.props.mask;
    };
    /**
     * @stable [17.06.2018]
     * @returns {string}
     */
    UniversalField.prototype.getFieldPattern = function () {
        return this.props.pattern;
    };
    /**
     * @stable [12.02.2019]
     * @param {AnyT} value
     * @param {boolean} returnDisplayValue
     * @returns {AnyT}
     */
    UniversalField.prototype.getDecoratedValue = function (value, returnDisplayValue) {
        if (returnDisplayValue === void 0) { returnDisplayValue = true; }
        var props = this.props;
        var displayValue = props.displayValue;
        var decoratedValue = this.decorateValueBeforeDisplaying(value);
        return util_1.isFn(displayValue)
            ? util_1.calc(displayValue, decoratedValue)
            : (returnDisplayValue
                ? (R.isNil(displayValue) ? decoratedValue : this.decorateValueBeforeDisplaying(displayValue))
                : decoratedValue);
    };
    /**
     * @stable [07.01.2018]
     * @param {AnyT} value
     * @returns {AnyT}
     */
    UniversalField.prototype.decorateValueBeforeDisplaying = function (value) {
        return value;
    };
    Object.defineProperty(UniversalField.prototype, "isInactive", {
        /**
         * @stable [28.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isFieldInactive(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isActive", {
        /**
         * @stable [28.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return !this.isInactive;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isDisabled", {
        /**
         * @stable [28.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isDisabled(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isReadOnly", {
        /**
         * @stable [29.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isReadOnly(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isChangeable", {
        /**
         * @stable [28.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isChangeable(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isFocused", {
        /**
         * @stable [30.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return this.isKeyboardUsed ? this.isKeyboardOpen() : util_1.isFocused(this.state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isVisible", {
        /**
         * @stable [25.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isVisible(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "inProgress", {
        /**
         * @stable [25.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return this.isFieldBusy();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [15.01.2020]
     * @returns {boolean}
     */
    UniversalField.prototype.isFieldBusy = function () {
        return util_1.inProgress(this.props);
    };
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    UniversalField.prototype.onFocus = function (event) {
        if (this.isFocusPrevented || this.isKeyboardUsed) {
            this.removeFocus(); // Prevent native keyboard opening during use of a synthetic keyboard
            if (this.isKeyboardUsed) {
                this.openVirtualKeyboard();
            }
        }
        else {
            this.setState({ focused: true });
        }
        var props = this.props;
        if (util_1.isFn(props.onFocus)) {
            props.onFocus(event);
        }
    };
    /**
     * @stable [30.10.2019]
     * @param {IFocusEvent} event
     */
    UniversalField.prototype.onBlur = function (event) {
        this.setState({ focused: false });
        var props = this.props;
        if (util_1.isFn(props.onBlur)) {
            props.onBlur(event);
        }
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getInputAttachmentElement = function () {
        return null;
    };
    /**
     * @stable [23.12.2019]
     * @react-native-compatible
     * @returns {React.ReactNode}
     */
    UniversalField.prototype.getDisplayValueElement = function () {
        var _this = this;
        return util_1.orNull(this.isDisplayValueOnly, function () { return _this.inProgress
            ? (_this.decoratedValue || _this.getWaitMessageElement())
            : _this.decoratedValue; });
    };
    /**
     * @stable [23.12.2019]
     * @react-native-compatible
     * @returns {React.ReactNode}
     */
    UniversalField.prototype.getWaitMessageElement = function () {
        return this.settings.messages.PLEASE_WAIT;
    };
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    UniversalField.prototype.openVirtualKeyboard = function () {
        var _this = this;
        if (this.isKeyboardOpen()) {
            return false;
        }
        this.setState({ keyboardOpen: true }, function () {
            if (util_1.isDef(_this.caretBlinkingTask)) {
                _this.caretBlinkingTask.start();
                /**
                 * @bugfix [08.05.2019]
                 * Need to refresh a caret position right after keyboard opening
                 */
                _this.refreshCaretPosition();
            }
        });
        return true;
    };
    /**
     * @stable [28.10.2019]
     */
    UniversalField.prototype.closeVirtualKeyboard = function () {
        this.setState({ keyboardOpen: false }, this.onCloseVirtualKeyboard);
    };
    /**
     * @stable [28.10.2019]
     */
    UniversalField.prototype.onCloseVirtualKeyboard = function () {
        if (util_1.isDef(this.caretBlinkingTask)) {
            this.caretBlinkingTask.stop();
        }
    };
    /**
     * @stable [28.10.2019]
     * @returns {boolean}
     */
    UniversalField.prototype.isKeyboardOpen = function () {
        return util_1.isKeyboardOpen(this.state);
    };
    Object.defineProperty(UniversalField.prototype, "isFocusPrevented", {
        /**
         * @stable [24.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isFocusPrevented(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isKeyboardUsed", {
        /**
         * @stable [24.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isKeyboardUsed(this.props);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isDisplayValueOnly", {
        /**
         * @stable [21.12.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isDisplayValueOnly(this.props);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getErrorMessageElement = function () {
        var _this = this;
        return util_1.orNull(this.props.errorMessageRendered !== false, function () { return _this.toMessageElement(_this.error, 'rac-field-error-text'); });
    };
    /**
     * @stable [03.09.2018]
     * @returns {JSX.Element}
     */
    UniversalField.prototype.getMessageElement = function () {
        var _this = this;
        return util_1.ifNotNilThanValue(this.props.message, function (message) { return _this.toMessageElement(message); });
    };
    Object.defineProperty(UniversalField.prototype, "error", {
        /**
         * @stable [03.09.2018]
         * @returns {string}
         */
        get: function () {
            return this.state.error;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [04.09.2018]
     */
    UniversalField.prototype.refreshCaretPosition = function () {
        var caretPosition = this.getCaretPosition();
        this.setState({ caretPosition: caretPosition });
        UniversalField.logger.debug("[$UniversalField][refreshCaretPosition] A caret position " + caretPosition + " for the field " + this.props.name + " had been refreshed.");
    };
    /**
     * @stable [04.09.2018]
     * @returns {number}
     */
    UniversalField.prototype.getCaretPosition = function () {
        // Need to implement
        return 0;
    };
    Object.defineProperty(UniversalField.prototype, "isSyntheticCursorUsed", {
        /**
         * @stable [30.10.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isSyntheticCursorUsed(this.props);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [21.12.2019]
     * @param value
     * @returns {boolean}
     */
    UniversalField.prototype.isValueDefined = function (value) {
        return util_1.isDef(value);
    };
    Object.defineProperty(UniversalField.prototype, "isValuePresent", {
        /**
         * @stable [21.12.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isValuePresent(this.value, this.emptyValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "isDisplayValueDefined", {
        /**
         * @stable [21.12.2019]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isDef(this.props.displayValue);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "defaultValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            return this.props.defaultValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "displayValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            return this.inProgress || !this.isValuePresent
                ? definition_1.FIELD_DISPLAY_EMPTY_VALUE
                : this.decoratedValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "decoratedValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            return this.getDecoratedValue(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "originalEmptyValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            return definition_1.FIELD_DISPLAY_EMPTY_VALUE;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalField.prototype, "emptyValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            var props = this.props;
            return util_1.isDef(props.emptyValue) ? props.emptyValue : this.originalEmptyValue;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [31.07.2018]
     * @param {AnyT} currentRawValue
     */
    UniversalField.prototype.onChangeValue = function (currentRawValue) {
        var actualFieldValue = util_1.buildActualFieldValue(__assign({}, this.props, { emptyValue: this.emptyValue, value: currentRawValue }));
        this.validateField(actualFieldValue);
        this.notifyAboutChanges(actualFieldValue);
        this.notifyFormAboutChanges(actualFieldValue);
    };
    /**
     * @stable [31.07.2018]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.validateField = function (rawValue) {
        // State value cannot take an undefined value then we should pass a null value at least
        this.setState({ error: this.validateValueAndSetCustomValidity(rawValue) });
    };
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.notifyFormAboutChanges = function (rawValue) {
        var props = this.props;
        util_1.ifNotNilThanValue(this.props.changeForm, function (changeForm) { return changeForm(props.name, rawValue); });
    };
    /**
     * @stable [16.11.2019]
     * @param {AnyT} rawValue
     */
    UniversalField.prototype.notifyAboutChanges = function (rawValue) {
        util_1.ifNotNilThanValue(this.props.onChange, function (onChange) { return onChange(rawValue); });
    };
    /**
     * @stable [09.11.2019]
     */
    UniversalField.prototype.setCaretVisibility = function () {
        var _this = this;
        this.setState(function (prevState) { return util_1.notNilValuesFilter({
            caretVisibility: !prevState.caretVisibility,
            caretPosition: util_1.ifNilThanValue(prevState.caretPosition, function () { return _this.getCaretPosition(); }),
        }); });
    };
    UniversalField.logger = ts_smart_logger_1.LoggerFactory.makeLogger('UniversalField');
    UniversalField.DEFAULT_CARET_BLINKING_FREQUENCY_TIMEOUT = 400;
    return UniversalField;
}(universal_component_1.UniversalComponent));
exports.UniversalField = UniversalField;
//# sourceMappingURL=universal-field.component.js.map