"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var R = require("ramda");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../../util");
var textfield_1 = require("../../field/textfield");
var menu_1 = require("../../menu");
var definition_1 = require("../../../definition");
var BaseSelect = /** @class */ (function (_super) {
    __extends(BaseSelect, _super);
    /**
     * @stable [30.11.2019]
     * @param {TProps} props
     */
    function BaseSelect(props) {
        var _this = _super.call(this, props) || this;
        _this.menuRef = React.createRef();
        _this.onClose = _this.onClose.bind(_this);
        _this.onFilterChange = _this.onFilterChange.bind(_this);
        _this.onSelect = _this.onSelect.bind(_this);
        _this.openMenu = _this.openMenu.bind(_this);
        _this.onDropDownClick = _this.onDropDownClick.bind(_this);
        if (_this.isExpandActionRendered) {
            _this.defaultActions = [
                { type: props.icon || definition_1.FieldActionTypesEnum.DROP_DOWN, onClick: _this.onDropDownClick }
            ].concat(_this.defaultActions);
        }
        if (_this.isQuickSelectionModeEnabled) {
            _this.quickFilterQueryTask = new util_1.DelayedTask(_this.onDelayedFilterChange.bind(_this), _this.delayTimeout);
        }
        return _this;
    }
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    BaseSelect.prototype.onChange = function (event) {
        _super.prototype.onChange.call(this, event);
        if (this.isQuickSelectionModeEnabled) {
            this.hideMenu();
            this.quickFilterQueryTask.start();
        }
    };
    /**
     * @stable [11.01.2020]
     * @param {Readonly<TProps extends IBaseSelectProps>} prevProps
     * @param {Readonly<TState extends IBaseSelectState>} prevState
     */
    BaseSelect.prototype.componentDidUpdate = function (prevProps, prevState) {
        var _this = this;
        _super.prototype.componentDidUpdate.call(this, prevProps, prevState);
        var props = this.props;
        if (this.isWaitingForData
            && R.isNil(prevProps.options)
            && !R.equals(props.options, prevProps.options)) {
            this.setState({ waitingForData: false }, function () {
                _this.renderAndShowMenu();
                if (util_1.isFn(props.onLoadDictionary)) {
                    props.onLoadDictionary(_this.filteredOptions);
                }
            });
        }
        // TODO Remove later
        var newValue = props.value;
        if (R.isNil(newValue) && !R.equals(newValue, prevProps.value)) {
            var $$cachedValue = this.state.$$cachedValue;
            if (!R.isNil($$cachedValue) && !R.equals($$cachedValue.value, newValue)) {
                // Need to reset the previous cached display value if the value has been cleared or replaced
                this.setState({ $$cachedValue: null });
            }
        }
    };
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    BaseSelect.prototype.onKeyEscape = function (event) {
        _super.prototype.onKeyEscape.call(this, event);
        this.hideMenu();
    };
    /**
     * @stable [30.11.2019]
     */
    BaseSelect.prototype.clearValue = function () {
        _super.prototype.clearValue.call(this);
        this.setState({ $$cachedValue: null });
    };
    /**
     * @stable [11.01.2020]
     */
    BaseSelect.prototype.openMenu = function () {
        if (this.inProgress || this.isMenuAlreadyRenderedAndOpened) {
            return;
        }
        var _a = this.props, bindDictionary = _a.bindDictionary, forceReload = _a.forceReload, onEmptyDictionary = _a.onEmptyDictionary, options = _a.options;
        var areOptionsUnavailable = R.isNil(options);
        if (forceReload || areOptionsUnavailable) {
            if (util_1.isFn(onEmptyDictionary)) {
                this.setState({ waitingForData: true }, function () { return onEmptyDictionary(bindDictionary); });
            }
            else {
                // Try open empty dialog menu to remote search
                this.renderAndShowMenu(!this.isQuickSelectionModeEnabled);
            }
        }
        else {
            this.renderAndShowMenu();
        }
    };
    /**
     * @stable [15.01.2020]
     * @returns {boolean}
     */
    BaseSelect.prototype.isFieldBusy = function () {
        return _super.prototype.isFieldBusy.call(this) || this.isWaitingForData;
    };
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    BaseSelect.prototype.onClick = function (event) {
        _super.prototype.onClick.call(this, event);
        this.openMenu();
    };
    /**
     * @stable [11.01.2020]
     * @returns {JSX.Element}
     */
    BaseSelect.prototype.getInputAttachmentElement = function () {
        var _this = this;
        return util_1.orNull(util_1.isMenuOpened(this.state), function () { return ( // To improve a performance
        React.createElement(menu_1.Menu, __assign({ ref: _this.menuRef, width: _this.menuWidth, options: _this.filteredOptions, onSelect: _this.onSelect, onFilterChange: _this.onFilterChange, onClose: _this.onClose }, _this.getMenuProps()))); });
    };
    /**
     * @stable [09.01.2020]
     * @returns {IMenuProps}
     */
    BaseSelect.prototype.getMenuProps = function () {
        return this.props.menuConfiguration;
    };
    Object.defineProperty(BaseSelect.prototype, "filteredOptions", {
        /**
         * @stable [30.11.2019]
         * @returns {ISelectOptionEntity[]}
         */
        get: function () {
            return this.options;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "isMenuAlreadyRenderedAndOpened", {
        /**
         * @stable [11.01.2020]
         * @returns {boolean}
         */
        get: function () {
            var menu = this.menu;
            return !R.isNil(menu) && menu.isOpen();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [11.01.2020]
     * @returns {string}
     */
    BaseSelect.prototype.getFieldClassName = function () {
        return util_1.joinClassName(_super.prototype.getFieldClassName.call(this), 'rac-base-select');
    };
    Object.defineProperty(BaseSelect.prototype, "originalEmptyValue", {
        /**
         * @stable [21.12.2019]
         * @returns {AnyT}
         */
        get: function () {
            return definition_1.FIELD_VALUE_TO_RESET;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [30.11.2019]
     * @param {ISelectOptionEntity} option
     */
    BaseSelect.prototype.onSelect = function (option) {
        var _this = this;
        this.setState({
            $$cachedValue: {
                value: option.value,
                label: String(option.label || option.value),
            },
        }, function () {
            _this.onChangeManually(option.value);
            _this.notifySelectOption(option);
        });
    };
    /**
     * @stable [11.01.2020]
     * @param {ISelectOptionEntity} option
     */
    BaseSelect.prototype.notifySelectOption = function (option) {
        if (!util_1.isDef(option)) {
            return;
        }
        var onSelect = this.props.onSelect;
        if (!util_1.isFn(onSelect)) {
            return;
        }
        onSelect(option);
    };
    /**
     * @stable [10.09.2019]
     */
    BaseSelect.prototype.onClose = function () {
        var props = this.props;
        if (util_1.isFn(props.onClose)) {
            props.onClose();
        }
    };
    /**
     * @stable [12.02.2019]
     * @param {AnyT} value
     * @returns {AnyT}
     */
    BaseSelect.prototype.getDecoratedValue = function (value) {
        var $$cachedValue = this.state.$$cachedValue;
        if (!R.isNil($$cachedValue)) {
            return _super.prototype.getDecoratedValue.call(this, $$cachedValue.label, false);
        }
        var selectedItem = R.find(function (option) { return option.value === value; }, this.options);
        return R.isNil(selectedItem)
            ? _super.prototype.getDecoratedValue.call(this, value)
            : (_super.prototype.getDecoratedValue.call(this, R.isNil(selectedItem.label)
                ? selectedItem.value
                : this.t(selectedItem.label), false));
    };
    Object.defineProperty(BaseSelect.prototype, "options", {
        /**
         * @stable [11.01.2020]
         * @returns {ISelectOptionEntity[]}
         */
        get: function () {
            return util_1.nvl(this.props.options, []);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "isExpandActionRendered", {
        /**
         * @stable [11.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isExpandActionRendered(this.props);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [11.01.2020]
     */
    BaseSelect.prototype.onDelayedFilterChange = function () {
        var _this = this;
        util_1.ifNotEmptyThanValue(this.value, function (value) { return _this.setState({ waitingForData: true }, function () { return _this.onFilterChange(value); }); });
    };
    /**
     * @stable [11.01.2020]
     * @param {IBaseEvent} event
     */
    BaseSelect.prototype.onDropDownClick = function (event) {
        this.domAccessor.cancelEvent(event);
        this.openMenu();
    };
    BaseSelect.prototype.onFilterChange = function (query) {
        var props = this.props;
        var onFilterChange = props.onFilterChange;
        var onDictionaryFilterChange = props.onDictionaryFilterChange;
        if (util_1.isFn(onFilterChange)) {
            onFilterChange(query);
        }
        if (util_1.isFn(onDictionaryFilterChange)) {
            onDictionaryFilterChange(props.bindDictionary, { payload: { query: query } });
        }
    };
    /**
     * @stable [11.01.2020]
     */
    BaseSelect.prototype.hideMenu = function () {
        if (this.isMenuAlreadyRenderedAndOpened) {
            this.setState({ menuRendered: false });
        }
    };
    /**
     * @stable [12.01.2020]
     * @param {boolean} force
     */
    BaseSelect.prototype.renderAndShowMenu = function (force) {
        var _this = this;
        if (force === void 0) { force = false; }
        if (!R.isEmpty(this.filteredOptions) || force) {
            this.setState({ menuRendered: true }, function () { return _this.menu.show(); });
        }
        else {
            BaseSelect.logger.debug('[$BaseSelect][renderAndShowMenu] The options are empty. The menu does not show.');
        }
    };
    Object.defineProperty(BaseSelect.prototype, "isQuickSelectionModeEnabled", {
        /**
         * @stable [11.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return !this.isFocusPrevented;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "isWaitingForData", {
        /**
         * @stable [12.01.2020]
         * @returns {boolean}
         */
        get: function () {
            return util_1.isWaitingForData(this.state);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "menuWidth", {
        /**
         * @stable [11.01.2020]
         * @returns {number}
         */
        get: function () {
            return this.domAccessor.getWidth(this.getSelf());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "delayTimeout", {
        /**
         * @stable [11.01.2020]
         * @returns {number}
         */
        get: function () {
            return util_1.nvl(this.props.delayTimeout, 1000);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "menu", {
        /**
         * @stable [30.11.2019]
         * @returns {IMenu}
         */
        get: function () {
            return this.menuRef.current;
        },
        enumerable: true,
        configurable: true
    });
    BaseSelect.logger = ts_smart_logger_1.LoggerFactory.makeLogger('BaseSelect');
    return BaseSelect;
}(textfield_1.BaseTextField));
exports.BaseSelect = BaseSelect;
//# sourceMappingURL=base-select.component.js.map