/// <reference types="react" />
/// <reference types="googlemaps" />
import { BaseComponent } from '../../base';
import { IGoogleMapsProps } from './google-maps.interface';
import { IGoogleMaps, IGoogleMapsHeatMapLayerConfigEntity, IGoogleMapsMarkerConfigEntity, IGoogleMapsOpenMenuContextEntity } from '../../../definition';
export declare class GoogleMaps extends BaseComponent<IGoogleMapsProps> implements IGoogleMaps, IGoogleMapsOpenMenuContextEntity {
    lat: number;
    lng: number;
    x: number;
    y: number;
    private clickEventListener;
    private dbClickEventListener;
    private googleMapsLibTask;
    private heatMapLayer;
    private map;
    private openMenuTask;
    private readonly markers;
    private readonly markersDragListeners;
    private readonly menuRef;
    /**
     * @stable [10.01.2020]
     * @param {IGoogleMapsProps} props
     */
    constructor(props: IGoogleMapsProps);
    /**
     * @stable [09.01.2020]
     */
    componentDidMount(): void;
    /**
     * @stable [10.01.2020]
     */
    componentWillUnmount(): void;
    /**
     * @stable [31.07.2018]
     * @returns {JSX.Element}
     */
    render(): JSX.Element;
    /**
     * @stable [31.07.2018]
     * @param {google.maps.MarkerOptions} cfg
     * @param {string} name
     * @returns {google.maps.Marker}
     */
    addMarker(cfg?: google.maps.MarkerOptions, name?: string): google.maps.Marker;
    /**
     * @stable [10.01.2020]
     * @param {IGoogleMapsMarkerConfigEntity} cfg
     */
    setMarkerState(cfg: IGoogleMapsMarkerConfigEntity): void;
    /**
     * @stable [04.03.2019]
     * @param {IGoogleMapsHeatMapLayerConfigEntity} cfg
     */
    addHeatMapLayer(cfg: IGoogleMapsHeatMapLayerConfigEntity): void;
    /**
     * @stable [10.01.2020]
     * @param {string} markerId
     * @param {google.maps.Marker} marker
     */
    private onMarkerDragEnd;
    /**
     * @stable [10.01.2020]
     * @param {IMenuItemEntity} item
     */
    private onMenuSelect;
    /**
     * @stable [10.01.2020]
     * @param {IGoogleMapsClickPayloadEntity} event
     */
    private onMapClick;
    /**
     * @stable [03.03.2019]
     * @param {IGoogleMapsClickPayloadEntity} event
     */
    private onMapDbClick;
    /**
     * @stable [10.01.2020]
     * @param {IGoogleMapsOpenMenuContextEntity} context
     */
    private openMenu;
    /**
     * @stable [31.07.2018]
     */
    private onGoogleMapsReady;
    /**
     * @stable [04.03.2019]
     * @param {ILatLngEntity} latLng
     * @param {number} zoom
     */
    private refreshMap;
    /**
     * @stable [09.01.2020]
     */
    private cancelGoogleMapsLibTask;
    /**
     * @stable [10.01.2020]
     * @returns {IMenu}
     */
    private readonly menu;
    /**
     * @stable [26.10.2018]
     * @returns {boolean}
     */
    private readonly haveMenuOptions;
    /**
     * @stable [04.03.2019]
     * @returns {IGoogleMapsSettingsEntity}
     */
    private readonly mapsSettings;
}
