import { IEffectsAction } from 'redux-effects-promise';
import { AnyT, EntityIdT, IDataWrapper, IEntity, IRemovedEntityWrapper } from '../../definitions.interface';
import { IModifyEntityPayloadWrapperEntity, IPagedEntity, ISelectEntityPayloadEntity, ISelectedWrapperEntity } from '../../definition';
export declare class ListActionBuilder {
    /**
     * @stable [13.11.2019]
     * @param {string} section
     * @returns {string}
     */
    static buildSortingDirectionChangeActionType(section: string): string;
    static buildChangeActionType(section: string): string;
    static buildFirstPageActionType(section: string): string;
    static buildLastPageActionType(section: string): string;
    static buildPreviousPageActionType(section: string): string;
    static buildNextPageActionType(section: string): string;
    /**
     * @stable [31.08.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLoadActionType(section: string): string;
    /**
     * @stable [31.08.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildCancelLoadActionType(section: string): string;
    /**
     * @stable [03.06.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLazyLoadActionType(section: string): string;
    /**
     * @stable [03.06.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLazyLoadDoneActionType(section: string): string;
    static buildUnTouchActionType(section: string): string;
    /**
     * @stable [31.08.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLoadDoneActionType(section: string): string;
    /**
     * @stable [31.08.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLoadErrorActionType(section: string): string;
    /**
     * @stable [04.06.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildLazyLoadErrorActionType(section: string): string;
    /**
     * @stable [07.12.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildSelectActionType(section: string): string;
    /**
     * @stable [07.12.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildSelectErrorActionType(section: string): string;
    static buildInsertActionType(section: string): string;
    /**
     * @stable [17.05.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildRemoveActionType(section: string): string;
    static buildUpdateActionType(section: string): string;
    /**
     * @stable [07.12.2018]
     * @param {string} section
     * @returns {string}
     */
    static buildCreateActionType(section: string): string;
    /**
     * @stable [26.10.2019]
     * @param {string} section
     * @returns {string}
     */
    static buildDeselectActionType(section: string): string;
    /**
     * @stable [19.03.2019]
     * @param {string} section
     * @returns {string}
     */
    static buildDestroyActionType(section: string): string;
    /**
     * @stable [19.03.2019]
     * @param {string} section
     * @returns {string}
     */
    static buildResetActionType(section: string): string;
    /**
     * @stable [19.10.2019]
     * @param {string} section
     * @param {ISelectedWrapperEntity<TEntity extends IEntity>} selectedWrapper
     * @returns {IEffectsAction}
     */
    static buildLazyLoadDoneAction<TEntity extends IEntity = IEntity>(section: string, selectedWrapper: ISelectedWrapperEntity<TEntity>): IEffectsAction;
    /**
     * @stable [15.03.2019]
     * @param {string} section
     * @returns {IEffectsAction}
     */
    static buildCreateAction(section: string): IEffectsAction;
    /**
     * @stable [03.06.2018]
     * @param {string} section
     * @param {ISelectedWrapperEntity} payload
     * @returns {IEffectsAction}
     */
    static buildLazyLoadAction(section: string, payload: ISelectEntityPayloadEntity): IEffectsAction;
    static buildDeselectAction(section: string): IEffectsAction;
    /**
     * @stable [19.03.2019]
     * @param {string} section
     * @returns {IEffectsAction}
     */
    static buildDestroyAction(section: string): IEffectsAction;
    /**
     * @stable [19.03.2019]
     * @param {string} section
     * @returns {IEffectsAction}
     */
    static buildResetAction(section: string): IEffectsAction;
    /**
     * @stable [17.05.2018]
     * @param {string} section
     * @param {IRemovedEntityWrapper} data
     * @returns {IEffectsAction}
     */
    static buildRemoveAction(section: string, data: IRemovedEntityWrapper): IEffectsAction;
    static buildUpdateAction(section: string, data?: IModifyEntityPayloadWrapperEntity): IEffectsAction;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @param {EntityIdT} id
     * @param {TChanges} changes
     * @returns {IEffectsAction}
     */
    static buildMergeItemAction<TChanges = IEntity>(section: string, id: EntityIdT, changes: TChanges): IEffectsAction;
    /**
     * @stable [11.12.2019]
     * @param section
     * @param data
     */
    static buildLoadAction<TData = AnyT>(section: string, data?: TData): IEffectsAction;
    /**
     * @stable [13.05.2019]
     * @param {string} section
     * @param {IDataWrapper<AnyT> & IPagedEntity} data
     * @returns {IEffectsAction}
     */
    static buildLoadDoneAction(section: string, data?: IDataWrapper<AnyT> & IPagedEntity): IEffectsAction;
    static buildNextPageAction(section: string, data?: AnyT): IEffectsAction;
    static buildPreviousPageAction(section: string, data?: AnyT): IEffectsAction;
    static buildFirstPageAction(section: string, data?: AnyT): IEffectsAction;
    static buildLastPageAction(section: string, data?: AnyT): IEffectsAction;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @returns {string}
     */
    static buildMergeActionType(section: string): string;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @param {IModifyEntityPayloadWrapperEntity} data
     * @returns {IEffectsAction}
     */
    static buildMergeAction(section: string, data?: IModifyEntityPayloadWrapperEntity): IEffectsAction;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @param {IModifyEntityPayloadWrapperEntity} data
     * @returns {IEffectsAction}
     */
    static buildMergePlainAction(section: string, data?: IModifyEntityPayloadWrapperEntity): IEffectsAction;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @param {ISelectEntityPayloadEntity} payload
     * @returns {IEffectsAction}
     */
    static buildSelectAction(section: string, payload: ISelectEntityPayloadEntity): IEffectsAction;
    /**
     * @stable [20.10.2019]
     * @param {string} section
     * @param {ISelectEntityPayloadEntity} payload
     * @returns {IEffectsAction}
     */
    static buildSelectPlainAction(section: string, payload: ISelectEntityPayloadEntity): IEffectsAction;
    /**
     * @stable [20.01.2019]
     * @param {string} section
     * @returns {IEffectsAction}
     */
    static buildCreatePlainAction(section: string): IEffectsAction;
}
