import * as React from 'react';
import { Store } from 'redux';
import { ErrorEventCategoriesEnum, IDomAccessor, IEnvironment, ILogManager, IRouter, IRoutesEntity, IUiFactory, IUiMessageConfigEntity, IUniversalStoreEntity } from '../../../definition';
import { ISettingsEntity } from '../../../settings';
export declare class UIDefaultFactory implements IUiFactory {
    private static readonly logger;
    private static readonly WIN_ERROR_ID;
    protected readonly domAccessor: IDomAccessor;
    protected readonly environment: IEnvironment;
    protected readonly logManager: ILogManager;
    protected readonly router: IRouter;
    protected readonly routes: IRoutesEntity;
    protected readonly settings: ISettingsEntity;
    protected readonly store: Store<IUniversalStoreEntity>;
    /**
     * @stable [07.10.2019]
     */
    constructor();
    /**
     * @stable [28.11.2019]
     * @param {IUiMessageConfigEntity} cfg
     * @returns {React.ReactNode}
     */
    makeMessage(cfg: IUiMessageConfigEntity): React.ReactNode;
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    makeWindowError(e: Error): Element;
    /**
     * @stable [02.12.2019]
     * @param {Error} e
     * @param {boolean} logging
     * @returns {React.ReactNode}
     */
    makeReactError(e: Error, logging?: boolean): React.ReactNode;
    /**
     * @stable [16.10.2019]
     * @param {Error} e
     * @param {Element} parent
     */
    protected makeWindowErrorBodyElement(e: Error, parent: Element): void;
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    protected makeReactErrorBodyElement(e: Error): JSX.Element;
    /**
     * @stable [07.10.2019]
     * @param {Error} error
     * @returns {string[]}
     */
    protected buildErrorMessages(error: Error): string[];
    /**
     * @stable [07.10.2019]
     * @returns {string}
     */
    protected readonly errorSystemInfoLine: string;
    /**
     * @stable [07.10.2019]
     * @param {Error} e
     * @returns {JSX.Element}
     */
    protected getErrorMessagesElement(e: Error): JSX.Element;
    /**
     * @stable [28.11.2019]
     * @returns {string[]}
     */
    protected getMessageWrapperClassNames(): string[];
    protected getMessageBodyClassNames(): string[];
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getErrorWrapperClassNames(): string[];
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getErrorClassNames(): string[];
    /**
     * @stable [16.10.2019]
     * @returns {JSX.Element}
     */
    protected makeReactRestartActionElement(): JSX.Element;
    /**
     * @stable [16.10.2019]
     * @param {Element} parent
     * @returns {Element}
     */
    protected makeWindowRestartActionElement(parent: Element): Element;
    /**
     * @stable [07.10.2019]
     * @returns {string[]}
     */
    protected getRestartActionClassName(): string[];
    /**
     * In case of an out memory error, it would be better to reload the page
     * @stable [16.10.2019]
     */
    protected onRestartAndReload(): Promise<void>;
    /**
     * @stable [29.11.2019]
     * @returns {Promise<void>}
     */
    protected onBeforeReload(): Promise<void>;
    /**
     * @stable [07.10.2019]
     * @param {ErrorEventCategoriesEnum} errorCategory
     * @param {Error} e
     */
    protected logError(errorCategory: ErrorEventCategoriesEnum, e: Error): void;
}
