"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var cond_1 = require("./cond");
var join_1 = require("./join");
var definitions_interface_1 = require("../definitions.interface");
/**
 * @stable [01.08.2018]
 * @param {IPlaceEntity} placeEntity
 * @returns {string}
 */
exports.asPlaceEntityFormattedName = function (placeEntity) {
    return cond_1.ifNotNilThanValue(placeEntity, function () { return join_1.join([
        join_1.join([placeEntity.streetNumber, placeEntity.street], ' '),
        placeEntity.city,
        placeEntity.region,
        placeEntity.country
    ], ', ') || placeEntity.formattedName; });
};
/**
 * @stable [01.08.2018]
 * @param {google.maps.GeocoderAddressComponent} cmp
 * @param {(cmp: google.maps.GeocoderAddressComponent) => TType} extractor
 * @returns {TType}
 */
exports.toPlaceDescription = function (cmp, extractor) {
    return cmp && extractor(cmp);
};
/**
 * @stable [10.01.2020]
 * @param {IPlaceEntity} placeEntity
 * @returns {string}
 */
exports.asPlaceParameter = function (placeEntity) {
    return cond_1.ifNotNilThanValue(placeEntity, function () { return placeEntity.formattedName; }, definitions_interface_1.UNDEF_SYMBOL);
};
/**
 * @stable [04.08.2018]
 * @param {google.maps.GeocoderResult | google.maps.places.PlaceResult} place
 * @returns {IPlaceEntity}
 */
exports.asPlaceEntity = function (place) {
    if (!Array.isArray(place.address_components)) {
        return null;
    }
    return __assign({}, cond_1.ifNotNilThanValue(place.geometry, function (geometry) { return ({
        lat: geometry.location.lat(),
        lng: geometry.location.lng(),
    }); }), { placeId: place.place_id, country: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('country'); }), function (cmp) { return cmp.long_name; }), region: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('administrative_area_level_1'); }), function (cmp) { return cmp.long_name; }), area: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('administrative_area_level_2'); }), function (cmp) { return cmp.long_name; }), city: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('locality'); }), function (cmp) { return cmp.long_name; }), street: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('route'); }), function (cmp) { return cmp.long_name; }), streetNumber: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('street_number'); }), function (cmp) { return cmp.long_name; }), zipCode: exports.toPlaceDescription(place.address_components.find(function (addr) { return addr.types.includes('postal_code'); }), function (cmp) { return cmp.long_name; }) });
};
//# sourceMappingURL=place.js.map